﻿using System;
using System.Collections.Generic;
using XMailAdminProxy;
using System.Text.RegularExpressions;
using System.IO;

public class XMUser
{
    [Serializable]
    public struct RedirectionsList
    {
        public List<string> forwards;
        public List<string> activeForwards;
    }

    [Serializable]
    public struct ExternalLinkObject
    {
        public string server;
        public string user;
    }

    [Serializable]
    public struct UserInfo
    {
        public string domain;
        public string user;
        public string pass;
        public string type;
    }
    
    public static void ChangePassword(string domainName, string userName, string userPassword)
    {
        try
        {
            string res = "";
            if (userPassword != "")
            {
                XMLayer xmLayer = new XMLayer();
                xmLayer = Utils.CreateXMLayer();
                xmLayer.Login();
                res = xmLayer.ChangePassword(domainName, userName, userPassword);
                xmLayer.Logout();
                if (res == "+00000 OK") res = "";
            }
        }
        catch (Exception)
        {
        }
    }

    public static void DeleteUser(string domainName, string userName)
    {
        try
        {
            XMLayer xm = new XMLayer();
            xm = Utils.CreateXMLayer();
            xm.Login();
            xm.DeleteUser(domainName, userName);
            xm.Logout();
        }
        catch (Exception)
        {
            throw;            
        }
    }

    public static void AddListUser(string domainName, string userName, string NewUserAddressID, MLPERMISSIONS permisson)
    {
        try
        {
            XMLayer xm = new XMLayer();
            xm = Utils.CreateXMLayer();
            xm.Login();
            xm.AddMailingListUser(domainName, userName, NewUserAddressID, permisson);
            xm.Logout();
        }
        catch (Exception)
        {
            throw;
        }
    }

    public static void DeleteListUser(string domainName, string userName, string address)
    {
        try
        {
            XMLayer xm = new XMLayer();
            xm = Utils.CreateXMLayer();
            xm.Login();
            xm.DeleteMailingListUser(domainName, userName, address.Trim());
            xm.Logout();
        }
        catch (Exception)
        {
            throw;
        }
    }

    public static void AddNewAlias(string domainName, string userName, string AccountAliasID)
    {
        try
        {
            XMLayer xm = new XMLayer();
            xm = Utils.CreateXMLayer();
            xm.Login();
            xm.AddAlias(domainName, userName, AccountAliasID);
            xm.Logout();
        }
        catch (Exception)
        {
            throw;
        }
    }

    public static void DeleteAlias(string domainName, string aliasNameForDelete)
    {
        XMLayer xm = new XMLayer();
        try
        {
            xm = Utils.CreateXMLayer();
            xm.Login();
            xm.DeleteAlias(domainName, aliasNameForDelete);
            xm.Logout();
        }
        catch (Exception)
        {
            throw;
        }
    }

    public static void SaveUserVariables(string domainName, string userName, string MaxMailboxSize, bool StoreMailLocaly)
    {
        try
        {
            List<string> varNames = new List<string>();
            List<string> varValues = new List<string>();

            varNames.Add("MaxMBSize");
            varValues.Add(MaxMailboxSize.Trim());

            varNames.Add("StoreMailLocaly");
            if (StoreMailLocaly)
                varValues.Add("1");
            else
                varValues.Add("0");

            XMLayer xm = new XMLayer();
            xm = Utils.CreateXMLayer();
            xm.Login();
            xm.SetUserVariable(domainName, userName, varNames.ToArray(), varValues.ToArray());
            xm.Logout();
        }
        catch (Exception)
        {
            throw;
        }
    }

    public static void AddExternalLink(string domainName, string userName, string ExternalPOP3Server, string ExternalUserName, string ExternalPassword, AUTHTYPE authType)
    {
        try
        {
            XMLayer xm = new XMLayer();
            xm = Utils.CreateXMLayer();
            xm.Login();
            xm.AddPop3ExternalLink(domainName, userName,ExternalPOP3Server,
                                                        ExternalUserName,
                                                        ExternalPassword, authType);
            xm.Logout();

        }
        catch (Exception)
        {
            throw;
        }
    }

    public static void DeleteExternalLink(string domainName, string userName, string extDomainName, string extUserName)
    {
        try
        {
            XMLayer xm = new XMLayer();
            xm = Utils.CreateXMLayer();
            xm.Login();
            xm.DeletePop3ExternalLink(domainName, userName, extDomainName.Trim(), extUserName.Trim());
            xm.Logout();
        }
        catch (Exception)
        {
            throw;
        }
    }

    public static void AdvancedSaveChanges(string domainName, string userName, string Advanced)
    {
        try
        {
            XMLayer xm = new XMLayer();
            xm = Utils.CreateXMLayer();
            xm.Login();
            string data = XMUser.GetMailProcTab(domainName, userName)[0];
            data += Advanced.Trim();
            data += "\r\n.\r\n";
            xm.SetMailProcTab(domainName, userName, data);
            xm.Logout();
        }
        catch (Exception)
        {
        }
    }

    public static UserInfo GetUser(string dName, string uName)
    {
        UserInfo user = new UserInfo();
        string line = string.Empty;
        string users = string.Empty;
        Regex re = new Regex("\\s+");

        try
        {
            XMLayer xmLayer = new XMLayer();
            xmLayer = Utils.CreateXMLayer();
            xmLayer.Login();
            users = xmLayer.ListUsers(dName, uName);
            xmLayer.Logout();

            using (StringReader sr = new StringReader(users))
            {
                while ((line = sr.ReadLine()) != null)
                {
                    line = line.Trim();
                    if (line == ".") break;
                    string[] us = re.Split(line);
                    user.domain = us[0].Trim(new char[] { '"' });
                    user.user = us[1].Trim(new char[] { '"' });
                    user.pass = us[2].Trim(new char[] { '"' });
                    user.type = us[3].Trim(new char[] { '"' });
                }
            }

        }
        catch (Exception)
        {
        }
        return user;
    }

    public static List<string> GetListMembers(string dName, string uName)
    {
        List<string> userList = new List<string>();
        try
        {
            XMLayer xm = new XMLayer();
            xm = Utils.CreateXMLayer();
            xm.Login();
            string raw = xm.ListMailingListUsers(dName, uName);
            xm.Logout();
            string line;
            using (StringReader sr = new StringReader(raw))
            {
                while ((line = sr.ReadLine()) != null)
                {
                    line = line.Trim();
                    if (line == ".") break;
                	string userAddress = extractMailingListUserInfo(line, "Address");
                	string userAccess = "";
                	string userAccessType = extractMailingListUserInfo(line, "AccessType");
					if (userAccessType.ToUpper().IndexOf("R") > -1)
					{
						userAccess = "Read";
					}
					if ((userAccessType.ToUpper().IndexOf("R") > -1) && 
						(userAccessType.ToUpper().IndexOf("W") > -1 || 
						userAccessType.ToUpper().IndexOf("A") > -1))
					{
						userAccess = "Read/Post";
					}
					userList.Add(userAddress + " (" + userAccess + ")");
                }
            }
        }
        catch (Exception)
        {
        }
        return userList;
    }

    public static string extractMailingListUserInfo(string complexUserName, string infoType)
    {
        try
        {
            Regex re = new Regex("\\s+");
            string[] parts = re.Split(complexUserName);

            if (infoType == "Address") return parts[0].Trim(new char[] { '"' });
            if (infoType == "AccessType") return parts[1].Trim(new char[] { '"' });
        }
        catch (Exception)
        {
        }
        return "undefined";
    }

    public static List<string> GetUsersAliases(string dName, string uName)
    {
        List<string> userAliases = new List<string>();
        try
        {
            XMLayer xm = new XMLayer();
            xm = Utils.CreateXMLayer();
            xm.Login();
            string raw = "";
            string line;
            raw = xm.ListAliases(dName, "*", uName);
            using (StringReader sr = new StringReader(raw))
            {
                while ((line = sr.ReadLine()) != null)
                {
                    line = line.Trim();
                    if (line == ".") break;
                    userAliases.Add(extractUsersAliasName(line));
                }
            }
            xm.Logout();
        }
        catch (Exception)
        {
        }
        return (userAliases.Count == 0) ? null : userAliases;
    }

    public static string extractUsersAliasName(string line)
    {
        try
        {
            Regex re = new Regex("\\s+");
            string[] parts = re.Split(line);
            return parts[1].Trim(new char[] { '"' });
        }
        catch (Exception)
        {
        }
        return "";
    }

    public static string[] GetUserVariables(string dName, string uName)
    {
        string[] userVariables = new string[5];
        for (int i = 0; i < userVariables.Length; i++) userVariables[i] = " ";
        try
        {
            XMLayer xm = new XMLayer();
            xm = Utils.CreateXMLayer();
            string raw = "";
            string line;
            xm.Login();
            raw = xm.ListUserVariables(dName, uName);
            xm.Logout();

            using (StringReader sr = new StringReader(raw))
            {
                while ((line = sr.ReadLine()) != null)
                {
                    line = line.Trim();
                    if (line == ".") break;
                    switch (extractUsersVariable(line, false))
                    {
                        case "RealName":
                            userVariables[0] = extractUsersVariable(line, true);
                            break;
                        case "HomePage":
                            userVariables[1] = extractUsersVariable(line, true);
                            break;
                        case "MaxMBSize":
                            userVariables[2] = extractUsersVariable(line, true);
                            break;
                        case "StoreMailLocaly":
                            userVariables[3] = extractUsersVariable(line, true);
                            break;
                        case "ClosedMailList":
                            userVariables[4] = extractUsersVariable(line, true);
                            break;
                        default:

                            break;
                    }
                }
                if (userVariables[3] == " ") userVariables[3] = "1";
            }
        }
        catch (Exception)
        {
        }
        return userVariables;
    }

    public static string extractUsersVariable(string line, bool flag)
    {
        try
        {
            Regex re = new Regex("\\s+");
            string[] parts = re.Split(line);
            if (flag)
                return parts[1].Trim(new char[] { '"' });
            else
                return parts[0].Trim(new char[] { '"' });
        }
        catch (Exception)
        {
        }
        return "";
    }
    public static RedirectionsList GetRedirections(string dName, string uName)//
    {
        RedirectionsList result = new RedirectionsList();
        result.forwards = new List<string>();
        result.activeForwards = new List<string>();
        try
        {
            XMLayer xm = new XMLayer();
            xm = Utils.CreateXMLayer();
            string raw = "";
            string line;
            xm.Login();
            raw = xm.GetMailProcTab(dName, uName);
            xm.Logout();
            using (StringReader sr = new StringReader(raw))
            {
                string[] data;
                int i = 0;
                while ((line = sr.ReadLine()) != null)
                {
                    line = line.Trim();
                    if (line == ".") break;
                    data = extractMailProc(line);
                    switch (data[0])
                    {
                        case "redirect":
                            for (i = 1; i < data.Length; i++)
                            {
                                result.forwards.Add(data[i]);
                            }
                            break;
                        case "lredirect":
                            for (i = 1; i < data.Length; i++)
                            {
                                result.activeForwards.Add(data[i]);
                            }
                            break;
                        default:
                            break;
                    }
                }
            }
        }
        catch (Exception)
        {
        }
        return result;
    }

    public static string[] extractMailProc(string line)
    {
        string[] parts = null;
        try
        {
            Regex re = new Regex("\\s+");
            parts = re.Split(line);
            for (int i = 0; i < parts.Length; i++) parts[i] = parts[i].Trim(new char[] { '"' });
        }
        catch (Exception)
        {
        }
        return parts;
    }
    public static List<ExternalLinkObject> GetExternalLink(string dName, string uName)
    {
        List<ExternalLinkObject> ExternalLink = new List<ExternalLinkObject>();
        XMLayer xm = new XMLayer();
        try
        {
            xm = Utils.CreateXMLayer();
            xm.Login();
            string raw = "";
            string line;
            raw = xm.ListPop3ExternalLink(dName, uName);
            xm.Logout();
            using (StringReader sr = new StringReader(raw))
            {
                while ((line = sr.ReadLine()) != null)
                {
                    line = line.Trim();
                    if (line == ".") break;
                    ExternalLinkObject elobj = new ExternalLinkObject();
                    elobj.server = extractExternalLink(line, "SERVER_NAME");
                    elobj.user = extractExternalLink(line, "USER_NAME");
                    ExternalLink.Add(elobj);
                }
            }
        }
        catch (Exception)
        {
        }
        return ExternalLink;
    }

    public static string extractExternalLink(string line, string param)
    {
        try
        {
            Regex re = new Regex("\\s+");
            string[] parts = re.Split(line);
            switch (param)
            {
                case "SERVER_NAME":
                    return parts[2].Trim(new char[] { '"' });
                case "USER_NAME":
                    return parts[3].Trim(new char[] { '"' });
                default:
                    break;
            }
        }
        catch (Exception)
        {
        }
        return "error";
    }

    public static string getName(string line, string param)
    {
        try
        {
            Regex re = new Regex("\\s+");
            string[] parts = re.Split(line);
            switch (param)
            {
                case "SERVER":
                    return parts[1].Trim(new char[] { '"' });
                case "USER":
                    return parts[0].Trim(new char[] { '"' });
                default:
                    break;
            }
        }
        catch (Exception)
        {
        }
        return "error";
    }

    public static string GetAdvanced(string dName, string uName)
    {
        string content = "";
        XMLayer xm = new XMLayer();
        try
        {
            xm = Utils.CreateXMLayer();
            xm.Login();
            string raw = "";
            string line;

            raw = xm.GetMailProcTab(dName, uName);
            xm.Logout();
            using (StringReader sr = new StringReader(raw))
            {
                string[] data;
                while ((line = sr.ReadLine()) != null)
                {
                    line = line.Trim();
                    if (line == ".") break;
                    data = extractMailProc(line);
                    switch (data[0])
                    {
                        case "mailbox":
                        case "redirect":
                        case "lredirect":
                            break;
                        default:
                            content += line + "\r\n";
                            break;
                    }
                }
            }
        }
        catch (XMailException)
        {
            throw;
        }
        return content;
    }

    public static string[] GetMailProcTab(string domainName, string userName)
    {
        string[] result = new string[2] { "", "" };
        XMLayer xm = new XMLayer();
        try
        {
            xm = Utils.CreateXMLayer();
            xm.Login();
            string raw = xm.GetMailProcTab(domainName, userName);
            xm.Logout();

            string line;
            result[0] = "";
            result[1] = "";
            string[] id;
            using (StringReader sr = new StringReader(raw))
            {
                while ((line = sr.ReadLine()) != null)
                {
                    line = line.Trim();
                    if (line == ".") break;
                    id = extractMailProc(line);
                    line = line.Trim();
                    switch (id[0])
                    {
                        case "mailbox":
                        case "redirect":
                        case "lredirect":
                            result[0] += line + "\r\n";
                            break;
                        default:
                            result[1] += line + "\r\n";
                            break;
                    }
                }
            }
        }
        catch (Exception)
        {
        }
        return result;
    }

    public static void AddDeleteRedirections(string domainName, string userName, string command, string element)
    {
        try
        {
            string data = "";
            string[] mailProcTabData;
            mailProcTabData = XMUser.GetMailProcTab(domainName, userName);

            string line;
            string head = "";

            string redirection = "";
            string lredirection = "";

            bool noRedirection = true;
            bool noLRedirection = true;

            string[] id;
            int lineIndex;
            StringReader reader = new StringReader(mailProcTabData[0]);
            switch (command)
            {
                case "AddToRedirection":
                case "AddToLRedirection":
                    lineIndex = 0;
                    while ((line = reader.ReadLine()) != null)
                    {
                        lineIndex++;
                        id = XMUser.extractMailProc(line);
                        line = line.Trim();
                        switch (id[0])
                        {
                            case "mailbox":
                                head = line + "\r\n";
                                break;
                            case "redirect":
                                noRedirection = false;
                                if (command == "AddToRedirection")
                                {
                                    redirection = line + '\t' + '\"' + element + '\"' + "\r\n";
                                }
                                else
                                {
                                    redirection = line + "\r\n";
                                }
                                break;
                            case "lredirect":
                                noLRedirection = false;
                                if (command == "AddToLRedirection")
                                {
                                    lredirection = line + '\t' + '\"' + element + '\"' + "\r\n";
                                }
                                else
                                {
                                    lredirection = line + "\r\n";
                                }
                                break;
                            default:
                                break;
                        }
                    }
                    if (head == "")
                    {
                        head = '\"' + "mailbox" + '\"' + "\r\n";
                    }
                    if (noRedirection)
                    {
                        if (command == "AddToRedirection") redirection = "redirect" + '\t' + '\"' + element + '\"' + "\r\n";
                    }
                    if (noLRedirection)
                    {
                        if (command == "AddToLRedirection") lredirection = "lredirect" + '\t' + '\"' + element + '\"' + "\r\n";
                    }
                    break;
                case "DeleteRedirection":
                case "DeleteLRedirection":
                    lineIndex = 0;
                    while ((line = reader.ReadLine()) != null)
                    {
                        lineIndex++;
                        id = XMUser.extractMailProc(line);
                        line = line.Trim();
                        switch (id[0])
                        {
                            case "mailbox":
                                head = line + "\r\n";
                                break;
                            case "redirect":
                                if (command == "DeleteRedirection")
                                {
                                    int index = line.IndexOf(element);
                                    string before = line.Substring(0, index - 2);
                                    string after = line.Substring(index + 1 + element.Length);
                                    redirection = before + after + "\r\n";
                                    if (redirection.Length == 10) redirection = "";
                                }
                                else
                                {
                                    redirection = line + "\r\n";
                                }
                                break;
                            case "lredirect":
                                if (command == "DeleteLRedirection")
                                {
                                    int index = line.IndexOf(element);
                                    string before = line.Substring(0, index - 2);
                                    string after = line.Substring(index + 1 + element.Length);
                                    lredirection = before + after + "\r\n";
                                    if (lredirection.Length == 11) lredirection = "";
                                }
                                else
                                {
                                    lredirection = line + "\r\n";
                                }
                                break;
                            default:
                                break;
                        }
                    }
                    break;
                default:
                    break;
            }
            data = head + redirection + lredirection + mailProcTabData[1];
            XMLayer xm = new XMLayer();
            xm = Utils.CreateXMLayer();
            xm.Login();
            xm.SetMailProcTab(domainName, userName, data);
            xm.Logout();
        }
        catch (Exception)
        {
            throw;
        }
    }

}
