using System;
using System.Configuration;
using System.Globalization;
using System.IO;
using System.Resources;
using System.Web.UI;
using System.Xml.XPath;

namespace WebMailLite
{
	public class WebmailResourceManager
	{
		protected CultureInfo _cultureInfo;
		protected ResourceManager _resManager;

		public CultureInfo Culture
		{
			get { return _cultureInfo; }
			set { _cultureInfo = value; }
		}

		public ResourceManager Manager
		{
			get { return _resManager; }
			set { _resManager = value; }
		}

		public WebmailResourceManager(string resourceDir)
		{
			_cultureInfo = CultureInfo.InvariantCulture;
			_resManager = ResourceManager.CreateFileBasedResourceManager("webmail", resourceDir, null);
		}

		public string GetString(string name)
		{
			try
			{
				string result = _resManager.GetString(name, _cultureInfo);
				if (result == null)
				{
					result = name;
				}
				return result;
			}
			catch (Exception)
			{
				return name;
			}
		}
	}

	/// <summary>
	/// Summary description for WebmailResourceManager.
	/// </summary>
	public class WebmailResourceManagerCreator : Control
	{
		public WebmailResourceManager CreateResourceManager()
		{
			WebmailSettings settings = (new WebMailSettingsCreator()).CreateWebMailSettings();
			string lang = settings.DefaultLanguage;
			if (this.Context.Session[Constants.sessionAccount] != null)
			{
				Account acct = this.Context.Session[Constants.sessionAccount] as Account;
				if (acct != null)
				{
					lang = settings.DefaultLanguage;
				}
			}

			WebmailResourceManager newManager = null;
			string culture = "";
			string langsXml = Path.Combine(Utils.GetDataFolderPath(), @"langs\langs.xml");
			if (File.Exists(langsXml))
			{
				XPathDocument xpathDoc = new XPathDocument(langsXml);
				XPathNavigator nav = xpathDoc.CreateNavigator();
				XPathNodeIterator langIter = nav.Select(string.Format("langs/lang[FriendlyName='{0}']/CultureName", lang));
				if (langIter.MoveNext())
				{
					culture = langIter.Current.Value;
				}
				else
				{
					culture = "";
				}
			}
			if (newManager != null)
			{
				newManager.Manager.ReleaseAllResources();
			}
			newManager = new WebmailResourceManager(Path.Combine(Utils.GetDataFolderPath(), @"langs"));
			newManager.Culture = new CultureInfo(culture);
			
			return newManager;
		}

	}
}
